import numpy as np
from numpy import linspace
from numpy.polynomial.polynomial import polyfit
from numpy.random import normal
from math import *
from random import random
from utility import *
import sys, resource, json
from simulation import *
import matplotlib.pyplot as plt

def sim(events, sensor=470, config="plates.json", res=.0051826, plt=None, toggle=None, title=None, use=True):
    plates=loadPlateFile(config)
    vals, positions, real_track, measured_track, risiduals = getSimulationData( plates, events, sensor, res=res, toggle=toggle, use=use)
    
#    rms=getRMS(risiduals)
    a = abs(sensor-plates[2]['position'])
    d = abs(plates[2]['position']-plates[1]['position'])
    r=a/d
    s = res
    S=s*sqrt(1+2*r*r)

    print("The given resolution is %.06f and the calculated S=%.06f"%(s,S))
    print("The scoring plane location is %s and plate 3 is at %s"%(sensor, plates[2]['position']))
    if plt is not None:
        plt.subplot(221)
        plt.plot(positions, real_track, marker='.', linestyle='None')
        plt.plot([sensor,sensor], [min(measured_track),max(measured_track)], 'r')
        plt.title("Real Track")
        plt.ylabel("Hit Location (mm)")

        plt.subplot(222)
        plt.plot(positions, measured_track,marker='.', linestyle='None')
        plt.plot([sensor,sensor], [min(measured_track),max(measured_track)], 'r')
        plt.title("Measured Track")
        plt.xlabel("Plate Positions (mm)")

        plt.subplot(223)
        plt.hist(vals,linspace(min(vals),max(vals),300))
        plt.xlabel("Veritle Axis, Hit Location (mm)")
        plt.ylabel("Number of Hits (count)")
        #plt.title("Hits Line of Best Fit at x=%s"%sensor)

        plt.subplot(224)
        plt.axis("off")
        plt.annotate(xy=(0,.8),s="%s Events"%events)
        plt.annotate(xy=(0,.7),s="Sensor is at %smm"%sensor)
        plt.annotate(xy=(0,.6),s="Resolution is %s"%res)
        plt.annotate(xy=(0,.5),s="Calculated rms at sensor is %.07f"%S)
        plt.annotate(xy=(0,.4),s="Plate 3 is located at %.07fmm"%plates[2]['position'])
        if title is not None: plt.annotate(xy=(.3,.5),s=title)
        
        plt.show()
    return S
plate_min=285
plate_max=750
test_range=linspace(plate_min,plate_max,20)
res=[]
events=50000
for test in test_range:
    res.append(sim(events, sensor=test))

plt.title("Scoring Plane Position vs RMS at the Scoring Plane %s events"%events)
plt.plot(test_range,res,linestyle='None', marker='o')
plt.xlabel("Scoring Plane Position (mm)")
plt.ylabel("RMS at the Scoring Plane (S)")
plt.show()
    



import matplotlib.pyplot as plt
from simulation import simulate
from numpy import linspace
import json

import argparse
parser = argparse.ArgumentParser(description="Utility for running simulation on a moving sensor.")
parser.add_argument('title', metavar="title", type=str, nargs='?', help="Title of the plot")
parser.add_argument('-b', metavar="bins", type=int, nargs='?', help="Number of points to collect.", default=20)
parser.add_argument('-e', metavar="events", type=int, nargs='?', help="Number of events to run.", default=300)
parser.add_argument('--min', metavar="xmin", type=float, nargs='?', help="Minimum x position for sensor.", default=305)
parser.add_argument('--max', metavar="xmax", type=float, nargs='?', help="Maximum x position for sensor.", default=635)
args=parser.parse_args()

plate_min=305
plate_max=635
test_range=linspace(plate_min,plate_max,20)

both_plates=[]
front_plates=[]
select_plates=[]
_both_plates=[]
_front_plates=[]
_select_plates=[]

events=10000
for test in test_range:
    use_coulomb=True
    front_plates.append(simulate(events=events,sensor=test,plt=None,toggle=(0,3), use=use_coulomb))
    select_plates.append(simulate(events=events,sensor=test,plt=None,toggle=(1,3), use=use_coulomb))
    both_plates.append(simulate(events=events,sensor=test,plt=None,toggle=None, use=use_coulomb))
    use_coulomb=False
    _front_plates.append(simulate(events=events,sensor=test,plt=None,toggle=(0,3), use=use_coulomb))
    _select_plates.append(simulate(events=events,sensor=test,plt=None,toggle=(1,3), use=use_coulomb))
    _both_plates.append(simulate(events=events,sensor=test,plt=None,toggle=None, use=use_coulomb))
    print("Finised sensor at %.04fmm." %test)

    
plt.subplot(121)
plt.plot(test_range,front_plates, linestyle='None', marker='o', label='Plates 1,2,3')
plt.plot(test_range,select_plates, linestyle='None', marker='o', label='Front Plates 2,3')
plt.plot(test_range,both_plates, linestyle='None', marker='o', label='Plates 1-6')
plt.xlabel("Scoring Plane Position (mm)")
plt.ylabel("RMS of Test Point in Line of Best Fits")
plt.ylim([0, max(front_plates+both_plates+select_plates)*1.05])
plt.legend(loc='upper left')
plt.grid(True, alpha=.2)
plt.title("Moving Scoring Plane %s Events"%events)

plt.subplot(122)
plt.plot(test_range,_front_plates, linestyle='None', marker='o', label='Plates 1,2,3')
plt.plot(test_range,_select_plates, linestyle='None', marker='o', label='Front Plates 2,3')
plt.plot(test_range,_both_plates, linestyle='None', marker='o', label='Plates 1-6')
plt.xlabel("Scoring Plane Position (mm)")
plt.ylabel("RMS of Test Point in Line of Best Fits")
plt.ylim([0, max(front_plates+both_plates+select_plates)*1.05])
plt.legend(loc='upper left')
plt.grid(True, alpha=.2)
plt.title("Moving Scoring Plane, No Coulomb With %s Events"%events)

file_name="moving_sensor.json"
with open(file_name,"w+") as f:
    output={
        'test_range': test_range.tolist(),
        'all_plates': both_plates,
        'front_plates': front_plates,
        'select_plates': select_plates
    }
    f.write(json.dumps(output))

print('wrote to file %s' % file_name)
plt.show()




import numpy as np
from numpy import linspace
from numpy.polynomial.polynomial import polyfit
from numpy.random import normal
from math import *
from random import random
from utility import *
import sys, resource, json
#7.6*10**-4
THCK=THICKNESS=.00076
BE=BEAMENERGY=12500 #12.5GeV
'''
| Notes on Radiation Thickness

From arXiv:1603.09669v2,
the plates are:
 5.5e-3cm Silicon; radlen 9.37cm
 5.0e-3cm Kapton;  radlen 28.6cm
Plates are .0055/9.37 + 0.0050/28.6 = 7.6x10^-4 radlen
'''

def getScatterRMS(thickness,velocity=BE):
    v=velocity
    out=(13.6)/(v) * sqrt(thickness) * (1+.088*log(thickness))
    return out

def getScatterAngle(thickness, use=True):
    if use is False: return 0
    rms=getScatterRMS(thickness)
    theta=normal(0,rms)
    return theta
    
def getPosition(positions,plates, use=True):
    track=[]
    theta=0
    previous_x=0
    y=0
    for x,plate in zip(positions,plates):
        d=x-previous_x
        y+=d*tan(theta)
        track.append(y)
        theta+=getScatterAngle(plate['radlen'], use)
        previous_x=x
    return track

def getMeasurement(real_track, plates, res=None):
    if res is None:
        return [ normal(y,plate['resolution']) for y,plate in zip(real_track, plates) ]
    else:
        return [ normal(y,res) for y in real_track ] 


def getSimulationData(plates, events=1, sensor=470, plt=None, res=None, toggle=None, use=True):
    if toggle is None: toggle=(0,len(plates))
    positions=[ plate['position'] for plate in plates ]
    real_tracks=[]
    measured_tracks=[]
    vals=[]
    risiduals=[]
    
    for e in range(events):
        start=e*len(plates)
        end=e*len(plates)+len(plates)
        real_tracks+=getPosition(positions, plates, use) # Start with a straight line
        measured_tracks+=getMeasurement(real_tracks[start:end],plates,res)
        if plt is not None and e%int(events/100) is 0:
            _plt=plt
        else:
            _plt=None
        vals.append(getTestPoint(sensor,positions[toggle[0]:toggle[1]],measured_tracks[start:end][toggle[0]:toggle[1]],plt=_plt))
        risiduals.append(getRisidual(positions[toggle[0]:toggle[1]],measured_tracks[start:end][toggle[0]:toggle[1]]))
    x=[]
    for i in range(events): x+=positions.copy()
    
    return vals, x, real_tracks, measured_tracks, risiduals

def simulate(events, sensor=470, config="plates.json", res=.0051826, plt=None, toggle=None, title=None, use=True):
    plates=loadPlateFile(config)
    vals, x, real_tracks, measured_tracks, risiduals = getSimulationData( plates, events, sensor, res=res, toggle=toggle, use=use, plt=plt)
    return getRMS(vals)

    




import matplotlib.pyplot as plt
from simulation import simulate
from numpy import linspace
import json
plate_min=305
plate_max=635
test_range=linspace(plate_min,plate_max,20)

select_plates=[]
_select_plates=[]
events=500
for test in test_range:
    use_coulomb=True
    plt.subplot(223)
    select_plates.append(simulate(events=events,sensor=test,toggle=None, use=use_coulomb,plt=plt))
    use_coulomb=False
    plt.subplot(224)
    _select_plates.append(simulate(events=events,sensor=test,toggle=None, use=use_coulomb, plt=plt))
    print("Finised sensor at %.04fmm." %test)

    
plt.subplot(221)
plt.plot(test_range,select_plates, linestyle='None', marker='o', label='Front Plates 2,3')
plt.xlabel("Scoring Plane Position (mm)")
plt.ylabel("RMS of Test Point in Line of Best Fits")
plt.ylim([0, max(select_plates)*1.05])
plt.legend(loc='upper left')
plt.grid(True, alpha=.2)
plt.title("Moving Scoring Plane %s Events"%events)

plt.subplot(222)
plt.plot(test_range,_select_plates, linestyle='None', marker='o', label='Front Plates 2,3')
plt.xlabel("Scoring Plane Position (mm)")
plt.ylabel("RMS of Test Point in Line of Best Fits")
plt.ylim([0, max(_select_plates)*1.05])
plt.legend(loc='upper left')
plt.grid(True, alpha=.2)
plt.title("Moving Scoring Plane, No Coulomb With %s Events"%events)

plt.show()



from numpy.polynomial.polynomial import polyfit
import json
from math import *

def cmToRad(input):
    return input/9.370 #Silicon had rad length of 9.37cm

def transform_to_origin(input):
    minimum=min(input)
    return [i-minimum for i in input]

def loadPlateFile(url):
    with open("plates.json") as f:
        plates=json.loads(f.read())
    return plates

def plotLine(plotter, inputs, outputs):
    b,m=polyfit(inputs,outputs, 1)
    plotter.plot(inputs,[m*x+b for x in inputs])
    return b,m

def getTestPoint(test_point,x,y,plt=None):
    b,m=polyfit(x,y, 1)
    if plt is not None:
        plt.plot(x,[m*i+b for i in x])
    return m*test_point+b
'''

a= sum 1
b=sum x
c=sumx^2
rms = sqrt((b/a)^2 - c/a)
'''
def getRMS(risiduals):
    a=len(risiduals)
    b=sum(risiduals)
    b=b*b
    c=sum([x*x for x in risiduals])/a
    return sqrt(c - (b/a)**2)

def getRisidual(x,y):
    b,m=polyfit(x,y, 1)
    test_point=m*x[1]+b
    return y[1]-test_point

# Plotting for json formatted like:
# [ [x_0,y_0],[x_1,y_1],...,[x_n,y_n] ]
def generalPloting(file_name, title, xlabel,ylabel,plt):
    data=None
    with open(file_name) as f:
        data=json.loads(f.read())
    x=[datum[0] for datum in data]
    y=[datum[1] for datum in data]
    plt.plot(x,y, linestyle="None", marker="o")
    b,m=plotLine(plt,x,y)
    plt.title(title)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.show()


def getComposition(materials):
    #materials should be an array with 2-tuples, (material, thickness)
    radlen=0
    for material in materials:
        radlen+=getRadlen(material[0],material[1])
    return radlen
    
KAPTON=28.6  #cm
SILICON=9.37 #cm
def getRadlen(thickness, material):
    material=material.lower()
    if material == "kapton":
        return thickness/KAPTON
    elif material == "silicon":
        return thickness/SILICON
    else:
        return None
